/* eslint-disable react/jsx-handler-names */
/* eslint-disable react/destructuring-assignment */
import React, { Component } from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import ObjectUtils from 'primereact/components/utils/ObjectUtils';
import Tooltip from 'primereact/tooltip';
import SelectButtonItem from './SelectButtonItem';

export class SelectButton extends Component {
	constructor(props) {
		super(props);
		this.state = {};
		this.onOptionClick = this.onOptionClick.bind(this);
	}

	componentDidMount() {
		if (this.props.tooltip) {
			this.renderTooltip();
		}
	}

	componentDidUpdate(prevProps) {
		if (prevProps.tooltip !== this.props.tooltip) {
			if (this.tooltip) this.tooltip.updateContent(this.props.tooltip);
			else this.renderTooltip();
		}
	}

	componentWillUnmount() {
		if (this.tooltip) {
			this.tooltip.destroy();
			this.tooltip = null;
		}
	}

	renderTooltip() {
		this.tooltip = new Tooltip({
			target: this.element,
			content: this.props.tooltip,
			options: this.props.tooltipOptions,
		});
	}

	onOptionClick(event) {
		if (this.props.disabled) {
			return;
		}

		const selected = this.isSelected(event.option);
		const optionValue = this.getOptionValue(event.option);
		let newValue;

		if (this.props.multiple) {
			const currentValue = this.props.value ? [...this.props.value] : [];

			if (selected)
				newValue = currentValue.filter(
					val => !ObjectUtils.equals(val, optionValue, this.props.dataKey)
				);
			else newValue = [...currentValue, optionValue];
		} else {
			if (selected) newValue = null;
			else newValue = optionValue;
		}

		if (this.props.onChange) {
			this.props.onChange({
				originalEvent: event.originalEvent,
				value: newValue,
				stopPropagation: () => {},
				preventDefault: () => {},
				target: {
					name: this.props.name,
					id: this.props.id,
					value: newValue,
				},
			});
		}
	}

	getOptionValue(option) {
		return this.props.optionLabel ? option : option.value;
	}

	getOptionLabel(option) {
		return this.props.optionLabel
			? ObjectUtils.resolveFieldData(option, this.props.optionLabel)
			: option.label;
	}

	isSelected(option) {
		let selected = false;
		const optionValue = this.getOptionValue(option);

		if (this.props.multiple) {
			if (this.props.value && this.props.value.length) {
				for (const val of this.props.value) {
					if (ObjectUtils.equals(val, optionValue, this.props.dataKey)) {
						selected = true;
						break;
					}
				}
			}
		} else {
			selected = ObjectUtils.equals(this.props.value, optionValue, this.props.dataKey);
		}

		return selected;
	}

	renderItems() {
		if (this.props.options && this.props.options.length) {
			return this.props.options.map(option => {
				const optionLabel = this.getOptionLabel(option);

				return (
					<SelectButtonItem
						key={optionLabel}
						label={optionLabel}
						className={option.className}
						option={option}
						onClick={this.onOptionClick}
						selected={this.isSelected(option)}
						tabIndex={this.props.tabIndex}
						disabled={this.props.disabled}
						ariaLabel={this.props.ariaLabel}
						ariaDescribedBy={this.props.ariaDescribedBy}
						ariaLabelledBy={this.props.ariaLabelledBy}
					/>
				);
			});
		} else {
			return null;
		}
	}

	render() {
		const className = classNames(
			'p-selectbutton p-buttonset p-component p-buttonset-3',
			this.props.className
		);
		const items = this.renderItems();

		return (
			<div id={this.props.id} ref={el => (this.element = el)} role="group">
				<div className={className} style={this.props.style}>
					{items}
				</div>
			</div>
		);
	}
}

SelectButton.defaultProps = {
	id: null,
	value: null,
	options: null,
	optionLabel: null,
	tabIndex: null,
	multiple: null,
	disabled: null,
	style: null,
	className: null,
	dataKey: null,
	tooltip: null,
	tooltipOptions: null,
	ariaLabelledBy: null,
	onChange: null,
};

SelectButton.propTypes = {
	ariaDescribedBy: PropTypes.string,
	ariaLabel: PropTypes.string,
	ariaLabelledBy: PropTypes.string,
	className: PropTypes.string,
	dataKey: PropTypes.string,
	disabled: PropTypes.bool,
	id: PropTypes.string,
	multiple: PropTypes.bool,
	name: PropTypes.string,
	onChange: PropTypes.func,
	optionLabel: PropTypes.string,
	options: PropTypes.array,
	required: PropTypes.bool,
	style: PropTypes.object,
	tabIndex: PropTypes.string,
	tooltip: PropTypes.string,
	tooltipOptions: PropTypes.object,
	value: PropTypes.any,
};

export default SelectButton;
